int datapoints = 20;
float[][] data;
Table table;
float offset;
PVector[] points;
PVector start, end; // Two points for the random line
float perpendicularLength1, perpendicularLength2; // Lengths of perpendicular lines
boolean fill = false;

float PriceMinValue, PriceMaxValue, VolumeMinValue, VolumeMaxValue;

void setup() {
  size(1080, 1080);
  loadData();
  //generateRandomData();
}
void draw() {
  plotGraph();
}

void loadData() {
  table = loadTable("TSM.csv", "header"); // Load the CSV file, "header" indicates the first row contains column headers
  int numRows = table.getRowCount(); // Get the number of rows in the table
  data = new float[numRows][2]; // Initialize the data array with 2 columns

  // Copy the "Name" and "Age" columns into the data array
  for (int i = 0; i < numRows; i++) {
    TableRow row = table.getRow(i);
    data[i][0] = row.getFloat("Adj Close ($)"); // Copy the "Name" column
    data[i][1] = row.getFloat("Volume (Shares)");  // Copy the "Age" column
  }
 PriceMinValue = data[0][0]; // Initialize minColumn1 with the first element
 PriceMaxValue = data[0][0]; // Initialize maxColumn1 with the first element
 VolumeMinValue = data[0][1]; // Initialize minColumn2 with the first element
 VolumeMaxValue = data[0][1]; // Initialize maxColumn2 with the first element

  for (int i = 1; i < data.length; i++) {
    if (data[i][0] < PriceMinValue) {
      PriceMinValue = data[i][0]; // Update minValue if a smaller value is found
    }
    if (data[i][0] > PriceMaxValue) {
      PriceMaxValue = data[i][0]; // Update maxValue if a larger value is found
    }

    if (data[i][1] < VolumeMinValue) {
      VolumeMinValue = data[i][1]; // Update minValue if a smaller value is found
    }
    if (data[i][1] > VolumeMaxValue) {
      VolumeMaxValue = data[i][1]; // Update maxValue if a larger value is found
    }
  }
  offset = width/float(data.length+1);
}

void generateRandomData() {
  for (int i = 0; i < datapoints; i++) {
    data[i][0] = random(1.0, 2.0); // Generate random price between 1.0 and 2.0
    data[i][1] = random(0.3, 1.3); // Generate random volume between 1.0 and 2.0
  }
}

void plotGraph() {
  background(255); // White background
  stroke(0); // Black lines
  if (fill) {
    noStroke();
    fill(0, 0, 0);
  } else {
    stroke(0);
    noFill();
  }
  float xSpacing = width / float(data.length+1); // Spacing between data points on x-axis

  // Draw data points and labels
  for (int i = 0; i < data.length-1; i++) {
    float yPrice = map(data[i][0], PriceMinValue, PriceMaxValue, height - height/12, height/12); // Map price to y-coordinate
    start = new PVector(offset + xSpacing * (i), yPrice);
    float yNextPrice = map(data[i+1][0], PriceMinValue, PriceMaxValue, height - height/12, height/12); // Map price to y-coordinate
    end = new PVector(offset + xSpacing * (i+1), yNextPrice);

    //draw the basic data line

    line(start.x, start.y, end.x, end.y);
    //draw perpendiculars
    PVector lineVector = end.copy().sub(start);
    PVector unitVector = lineVector.copy().normalize();
    float perpendicularLengthStart = map(data[i][1], VolumeMinValue, VolumeMaxValue, 6, 36 ); // Adjust the range as needed
    float perpendicularLengthEnd =  map(data[i+1][1], VolumeMinValue, VolumeMaxValue, 6, 36  ); // Adjust the range as needed

    PVector perpendicularStart1 = start.copy().add(unitVector.copy().rotate(HALF_PI).mult(-perpendicularLengthStart));
    PVector perpendicularEnd1 = start.copy().add(unitVector.copy().rotate(HALF_PI).mult(perpendicularLengthStart));
    PVector perpendicularStart2 = end.copy().add(unitVector.copy().rotate(HALF_PI).mult(-perpendicularLengthEnd));
    PVector perpendicularEnd2 = end.copy().add(unitVector.copy().rotate(HALF_PI).mult(perpendicularLengthEnd));

    // Draw the perpendicular lines

    line(perpendicularStart1.x, perpendicularStart1.y, perpendicularEnd1.x, perpendicularEnd1.y);
    line(perpendicularStart2.x, perpendicularStart2.y, perpendicularEnd2.x, perpendicularEnd2.y);
    //Draw the knee circles
    ellipse(start.x, start.y, perpendicularLengthStart*2, perpendicularLengthStart*2);
    //Draw the last knee circles
    if (i == data.length-2) {
      ellipse(end.x, end.y, perpendicularLengthEnd*2, perpendicularLengthEnd*2);
    }
    //Draw the polygon

    drawPolygon(perpendicularEnd1.x, perpendicularEnd1.y, perpendicularEnd2.x, perpendicularEnd2.y, perpendicularStart2.x, perpendicularStart2.y, perpendicularStart1.x, perpendicularStart1.y);
  }
}

void drawPolygon(float ax, float ay, float bx, float by, float cx, float cy, float dx, float dy) {
  points = new PVector[4];
  points[0] = new PVector(ax, ay);
  points[1] = new PVector(bx, by);
  points[2] = new PVector(cx, cy);
  points[3] = new PVector(dx, dy);
  if (isConvexQuadrilateral(points)) {
    beginShape();
    for (int i = 0; i < 4; i++) {
      vertex(points[i].x, points[i].y);
    }
    endShape(CLOSE); // CLOSE parameter connects the first and last points to close the polygon
  }
  if (!isConvexQuadrilateral(points)) {
    points[0] = new PVector(ax, ay);
    points[1] = new PVector(cx, cy);
    points[2] = new PVector(bx, by);
    points[3] = new PVector(dx, dy);
    beginShape();
    for (int i = 0; i < 4; i++) {
      vertex(points[i].x, points[i].y);
    }
    endShape(CLOSE); // CLOSE parameter connects the first and last points to close the polygon
  }
}

boolean isConvexQuadrilateral(PVector[] vertices) {
  // Check if the cross products of consecutive edges have the same sign
  boolean sign = false;

  for (int i = 0; i < 4; i++) {
    PVector p1 = vertices[i];
    PVector p2 = vertices[(i + 1) % 4];
    PVector p3 = vertices[(i + 2) % 4];

    float crossProduct = (p2.x - p1.x) * (p3.y - p1.y) - (p2.y - p1.y) * (p3.x - p1.x);

    if (i == 0) {
      sign = crossProduct > 0;
    } else if (crossProduct > 0 != sign) {
      return false; // Cross products have different signs, not convex
    }
  }

  return true; // All cross products have the same sign, convex
}

void keyPressed() {
  if (key == 'f' || key == 'F') {
    fill = !fill;
    println("pressed f");
  }
  if (key == 'r' || key == 'R') {
    generateRandomData();
    println("pressed r");
  }
  if (key == 's' || key == 'S') {
    saveFrame("niemeyer_graph_#####.jpg");
    println("saved Frame");
  }
}
